@@ -1,4 +1,4 @@
-/* $Header: /home/hyperion/mu/christos/src/sys/tcsh-6.00/RCS/sh.char.c,v 3.1 1991/07/15 19:37:24 christos Exp $ */
+/* $Header: /home/hyperion/mu/christos/src/sys/tcsh-6.00/RCS/sh.char.c,v 3.3 1991/10/12 04:23:51 christos Exp $ */
 /*
  * sh.char.c: Character classification tables
  */
@@ -34,8 +34,9 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  */
-#include "config.h"
-RCSID("$Id: sh.char.c,v 3.1 1991/07/15 19:37:24 christos Exp $")
+#include "sh.h"
+
+RCSID("$Id: sh.char.c,v 3.3 1991/10/12 04:23:51 christos Exp $")
 
 #include "sh.char.h"
 
@@ -113,7 +114,7 @@ unsigned short _cmap[256] = {
 	_ESC,		0,		0,		0,
 
 /*	`		a		b		c	*/
-  _Q1|_GLOB|_CMD|_META,	_LET|_LOW|_XD,	_LET|_LOW|_XD,	_LET|_LOW|_XD,
+  _Q1|_GLOB|_META,	_LET|_LOW|_XD,	_LET|_LOW|_XD,	_LET|_LOW|_XD,
 
 /*	d		e		f		g	*/
 	_LET|_LOW|_XD,	_LET|_LOW|_XD,	_LET|_LOW|_XD,	_LET|_LOW,
