@@ -1,6 +1,6 @@
-/* $Header: /home/hyperion/mu/christos/src/sys/tcsh-6.00/RCS/sh.decls.h,v 3.2 1991/07/15 19:37:24 christos Exp $ */
+/* $Header: /afs/sipb.mit.edu/project/tcsh/beta/tcsh-6.00-b3/RCS/sh.decls.h,v 1.2 91/10/02 03:47:43 marc Exp $ */
 /*
- * sh.decls	 External declarations from sh*.c
+ * sh.decls.h	 External declarations from sh*.c
  */
 /*-
  * Copyright (c) 1980, 1991 The Regents of the University of California.
@@ -86,8 +86,8 @@ extern	void		  heredoc	__P((Char *));
 /*
  * sh.err.c
  */
-extern	void		  seterror	__P((int, ...));
-extern	void		  stderror	__P((int, ...));
+extern	void		  seterror	__P((unsigned int, ...));
+extern	void		  stderror	__P((unsigned int, ...));
 
 /*
  * sh.exec.c
@@ -103,11 +103,12 @@ extern	void		  xechoit	__P((Char **));
 extern	int		  iscommand	__P((Char *));
 extern	int		  executable	__P((Char *, Char *, bool));
 extern	void		  tellmewhat	__P((struct wordent *));
+extern	void		  dowhere	__P((Char **, struct command *));
 
 /*
  * sh.exp.c
  */
-extern	int	 	  exp		__P((Char ***));
+extern	int	 	  expr		__P((Char ***));
 extern	int		  exp0		__P((Char ***, bool));
 
 /*
@@ -188,10 +189,8 @@ extern	void	 	  savehist	__P((struct wordent *));
  * sh.lex.c
  */
 extern	void		  addla		__P((Char *));
-extern	void		  bseek		__P((off_t));
-#ifndef btell
-extern	off_t		  btell		__P((void));
-#endif
+extern	void		  bseek		__P((struct Ain *));
+extern	void		  btell		__P((struct Ain *));
 extern	void		  btoeof	__P((void));
 extern	void		  copylex	__P((struct wordent *, 
 					     struct wordent *));
