@@ -1,4 +1,4 @@
-/* $Header: /home/hyperion/mu/christos/src/sys/tcsh-6.00/RCS/sh.dol.c,v 3.2 1991/07/24 17:38:12 christos Exp $ */
+/* $Header: /home/hyperion/mu/christos/src/sys/tcsh-6.00/RCS/sh.dol.c,v 3.6 1991/10/21 17:24:49 christos Exp $ */
 /*
  * sh.dol.c: Variable substitutions
  */
@@ -34,11 +34,10 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  */
-#include "config.h"
-RCSID("$Id: sh.dol.c,v 3.2 1991/07/24 17:38:12 christos Exp $")
-
 #include "sh.h"
 
+RCSID("$Id: sh.dol.c,v 3.6 1991/10/21 17:24:49 christos Exp $")
+
 /*
  * C shell
  */
@@ -81,6 +80,7 @@ static Char dolmod[MAXMOD];	/* : modifier character */
 static int dolnmod;		/* Number of modifiers */
 #endif /* COMPAT */
 static int dolmcnt;		/* :gx -> 10000, else 1 */
+static int dolwcnt;		/* :ax -> 10000, else 1 */
 
 static	void	 Dfix2		__P((Char **));
 static	Char 	*Dpack		__P((Char *, Char *));
@@ -398,11 +398,12 @@ Dgetdol()
     bool    dimen = 0, bitset = 0;
     char    tnp;
     Char    wbuf[BUFSIZ];
+    static Char *dolbang = NULL;
 
 #ifdef COMPAT
-    dolmod = dolmcnt = 0;
+    dolmod = dolmcnt = dolwcnt = 0;
 #else
-    dolnmod = dolmcnt = 0;
+    dolnmod = dolmcnt = dolwcnt = 0;
 #endif /* COMPAT */
     c = sc = DgetC(0);
     if (c == '{')
@@ -413,6 +414,16 @@ Dgetdol()
 	bitset++, c = DgetC(0);	/* $? tests existence */
     switch (c) {
 
+    case '!':
+	if (dimen || bitset)
+	    stderror(ERR_SYNTAX);
+	if (backpid != 0) {
+	    if (dolbang) 
+		xfree((ptr_t) dolbang);
+	    setDolp(dolbang = putn(backpid));
+	}
+	goto eatbrac;
+
     case '$':
 	if (dimen || bitset)
 	    stderror(ERR_SYNTAX);
@@ -624,10 +635,46 @@ fixDolMod()
 #ifndef COMPAT
 	do {
 #endif /* COMPAT */
-	    c = DgetC(0), dolmcnt = 1;
-	    if (c == 'g')
-		c = DgetC(0), dolmcnt = 10000;
-	    if (!any("htrqxe", c))
+	    c = DgetC(0), dolmcnt = 1, dolwcnt = 1;
+	    if (c == 'g' || c == 'a') {
+		if (c == 'g')
+		    dolmcnt = 10000;
+		else
+		    dolwcnt = 10000;
+		c = DgetC(0);
+	    }
+	    if ((c == 'g' && dolmcnt != 10000) || 
+		(c == 'a' && dolwcnt != 10000)) {
+		if (c == 'g')
+		    dolmcnt = 10000;
+		else
+		    dolwcnt = 10000;
+		c = DgetC(0); 
+	    }
+
+	    if (c == 's') {	/* [eichin:19910926.0755EST] */
+		int delimcnt = 2;
+		int delim = DgetC(0);
+		dolmod[dolnmod++] = c;
+		dolmod[dolnmod++] = delim;
+		
+		if (!delim || letter(delim)
+		    || Isdigit(delim) || any(" \t\n", delim)) {
+		    seterror(ERR_BADSUBST);
+		    break;
+		}	
+		while ((c = DgetC(0)) != (-1)) {
+		    dolmod[dolnmod++] = c;
+		    if(c == delim) delimcnt--;
+		    if(!delimcnt) break;
+		}
+		if(delimcnt) {
+		    seterror(ERR_BADSUBST);
+		    break;
+		}
+		continue;
+	    }
+	    if (!any("htrqxes", c))
 		stderror(ERR_BADMOD, c);
 #ifndef COMPAT
 	    dolmod[dolnmod++] = c;
@@ -667,16 +714,78 @@ setDolp(cp)
     dp = domod(cp, dolmod);
 #else
     dp = cp = Strsave(cp);
-    for (i = 0; i < dolnmod; i++)
-	if ((dp = domod(cp, dolmod[i]))) {
-	    xfree((ptr_t) cp);
-	    cp = dp;
-	    dolmcnt--;
-	}
-	else {
+    for (i = 0; i < dolnmod; i++) {
+	/* handle s// [eichin:19910926.0510EST] */
+	if(dolmod[i] == 's') {
+	    int delim;
+	    Char *lhsub, *rhsub, *np;
+	    size_t lhlen = 0, rhlen = 0;
+	    int didmod = 0;
+		
+	    delim = dolmod[++i];
+	    if (!delim || letter(delim)
+		|| Isdigit(delim) || any(" \t\n", delim)) {
+		seterror(ERR_BADSUBST);
+		break;
+	    }
+	    lhsub = &dolmod[++i];
+	    while(dolmod[i] != delim && dolmod[++i]) {
+		lhlen++;
+	    }
+	    dolmod[i] = 0;
+	    rhsub = &dolmod[++i];
+	    while(dolmod[i] != delim && dolmod[++i]) {
+		rhlen++;
+	    }
+	    dolmod[i] = 0;
+
+	    do {
+		dp = Strstr(cp, lhsub);
+		if (dp) {
+		    np = (Char *) xmalloc((size_t)
+					  ((Strlen(cp) + 1 - lhlen + rhlen) *
+					  sizeof(Char)));
+		    (void) Strncpy(np, cp, dp - cp);
+		    (void) Strcpy(np + (dp - cp), rhsub);
+		    (void) Strcpy(np + (dp - cp) + rhlen, dp + lhlen);
+
+		    xfree((ptr_t) cp);
+		    dp = cp = np;
+		    didmod = 1;
+		} else {
+		    /* should this do a seterror? */
+		    break;
+		}
+	    }
+	    while (dolwcnt == 10000);
+	    /*
+	     * restore dolmod for additional words
+	     */
+	    dolmod[i] = rhsub[-1] = delim;
+	    if (didmod)
+		dolmcnt--;
+	    else
+		break;
+        } else {
+	    int didmod = 0;
+
+	    do {
+		if ((dp = domod(cp, dolmod[i]))) {
+		    xfree((ptr_t) cp);
+		    cp = dp;
+		    didmod = 1;
+		}
+		else
+		    break;
+	    }
+	    while (dolwcnt == 10000);
 	    dp = cp;
-	    break;
+	    if (didmod)
+		dolmcnt--;
+	    else
+		break;
 	}
+    }
 #endif /* COMPAT */
 
     if (dp) {
