@@ -1,4 +1,4 @@
-/* $Header: /home/hyperion/mu/christos/src/sys/tcsh-6.00/RCS/sh.h,v 3.11 1991/08/05 23:02:13 christos Exp $ */
+/* $Header: /home/hyperion/mu/christos/src/sys/tcsh-6.00/RCS/sh.h,v 3.19 1991/10/22 06:52:57 christos Exp $ */
 /*
  * sh.h: Catch it all globals and includes file!
  */
@@ -37,7 +37,23 @@
 #ifndef _h_sh
 #define _h_sh
 
+/* This is separated instead of a #else of above because makedepend is
+easily confused. */
 
+#ifndef CONFIGH
+# define CONFIGH "config.h"
+#endif
+
+/*
+ * Avoid cpp bugs (CONFIGH is always defined at this point)
+ */
+#ifdef CONFIGH
+# include CONFIGH
+#endif
+
+#ifndef EXTERN
+# define EXTERN extern
+#endif /* EXTERN */
 /*
  * Sanity
  */
@@ -102,7 +118,24 @@ typedef int sigret_t;
 #define	xexit(n)	done(n)
 #endif 
 
+#ifdef cray
+# define word word_t           /* sys/types.h defines word.. bad move! */
+#endif
+
 #include <sys/types.h>
+
+#ifdef cray
+# undef word
+#endif 
+
+/*
+ * This macro compares the st_dev field of struct stat. On aix on ibmESA
+ * st_dev is a structure, so comparison does not work. 
+ */
+#ifndef DEV_DEV_COMPARE
+# define DEV_DEV_COMPARE(x,y)   ((x) == (y))
+#endif /* DEV_DEV_COMPARE */
+
 #ifdef _SEQUENT_
 # include <sys/procstats.h>
 #endif /* _SEQUENT_ */
@@ -119,26 +152,20 @@ typedef int sigret_t;
 
 #ifdef BSDTIMES
 # include <sys/time.h>
-# include <sys/resource.h>
+# if SVID>3
+#  include "/usr/ucbinclude/sys/resource.h"
+# else
+#  include <sys/resource.h>
+# endif /* SVID>3 */
 #endif /* BSDTIMES */
 
 #ifndef POSIX
 # ifdef TERMIO
-#  ifdef _IBMR2
- /* Aix redefines NOFLSH when _BSD_INCLUDES is defined! */
-#   undef _BSD
-#   undef _BSD_INCLUDES
-#  endif /* _IBMR2 */
 #  include <termio.h>
-#  ifdef _IBMR2
- /* but we need it.  */
-#   define _BSD
-#   define _BSD_INCLUDES
-#  endif /* _IBMR2 */
-# else
+# else /* SGTTY */
 #  include <sgtty.h>
 # endif /* TERMIO */
-#else				/* POSIX */
+#else /* POSIX */
 # include <termios.h>
 #endif /* POSIX */
 
@@ -174,11 +201,11 @@ extern int setpgrp();
 # include <limits.h>
 #endif /* POSIX */
 
-#if SVID > 0
+#if SVID > 0 || defined(_IBMR2)
 # if !defined(pyr) || !defined(aiws)
 #  include <time.h>
 # endif /* !aiws || !pyr */
-#endif /* SVID > 0 */
+#endif /* SVID > 0 ||  _IBMR2 */
 
 #if !(defined(sun) && defined(TERMIO))
 # include <sys/ioctl.h>
@@ -267,17 +294,29 @@ typedef union {
 # define calloc		lint_calloc
 #endif 
 
-#ifdef SYSMALLOC
-# define xmalloc(i)  	Malloc(i)
-# define xrealloc(p, i)	Realloc(p, i)
-# define xcalloc(n, s)	Calloc(n, s)
-# define xfree(p)    	Free(p)
+#ifdef MDEBUG
+extern memalign_t	DebugMalloc	__P((unsigned, char *, int));
+extern memalign_t	DebugRealloc	__P((ptr_t, unsigned, char *, int));
+extern memalign_t	DebugCalloc	__P((unsigned, unsigned, char *, int));
+extern void		DebugFree	__P((ptr_t, char *, int));
+# define xmalloc(i)  	DebugMalloc(i, __FILE__, __LINE__)
+# define xrealloc(p, i)((p) ? DebugRealloc(p, i, __FILE__, __LINE__) : \
+			      DebugMalloc(i, __FILE__, __LINE__))
+# define xcalloc(n, s)	DebugCalloc(n, s, __FILE__, __LINE__)
+# define xfree(p)    	if (p) DebugFree(p, __FILE__, __LINE__); else
 #else
+# ifdef SYSMALLOC
+#  define xmalloc(i)  	Malloc(i)
+#  define xrealloc(p, i)Realloc(p, i)
+#  define xcalloc(n, s)	Calloc(n, s)
+#  define xfree(p)    	Free(p)
+# else
 # define xmalloc(i)  	malloc(i)
 # define xrealloc(p, i)	realloc(p, i)
 # define xcalloc(n, s)	calloc(n, s)
 # define xfree(p)    	free(p)
-#endif /* SYSMALLOC */
+# endif /* SYSMALLOC */
+#endif /* MDEBUG */
 #include "sh.char.h"
 #include "sh.err.h"
 #include "sh.dir.h"
@@ -297,7 +336,7 @@ typedef union {
  */
 
 #define SIGN_EXTEND_CHAR(a) \
-	((a) & 0x80 ? ((int) (a)) | 0xffffff00 : ((int) a) & 0x000000ff)
+	((int) ((a) & 0x80 ? ((int) (a)) | 0xffffff00 : ((int) a) & 0x000000ff))
 
 
 
@@ -321,67 +360,68 @@ typedef union {
 /*
  * Global flags
  */
-bool    chkstop;		/* Warned of stopped jobs... allow exit */
+EXTERN bool    chkstop;		/* Warned of stopped jobs... allow exit */
 
 #ifndef FIOCLEX
-bool    didcch;			/* Have closed unused fd's for child */
+EXTERN bool    didcch;		/* Have closed unused fd's for child */
 #endif 
 
-bool    didfds;			/* Have setup i/o fd's for child */
-bool    doneinp;		/* EOF indicator after reset from readc */
-bool    exiterr;		/* Exit if error or non-zero exit status */
-bool    child;			/* Child shell ... errors cause exit */
-bool    haderr;			/* Reset was because of an error */
-bool    intty;			/* Input is a tty */
-bool    intact;			/* We are interactive... therefore prompt */
-bool    justpr;			/* Just print because of :p hist mod */
-bool    loginsh;		/* We are a loginsh -> .login/.logout */
-bool    neednote;		/* Need to pnotify() */
-bool    noexec;			/* Don't execute, just syntax check */
-bool    pjobs;			/* want to print jobs if interrupted */
-bool    setintr;		/* Set interrupts on/off -> Wait intr... */
-bool    timflg;			/* Time the next waited for command */
-bool    havhash;		/* path hashing is available */
-bool    editing;		/* doing filename expansion and line editing */
-bool    bslash_quote;		/* PWP: tcsh-style quoting?  (in sh.c) */
-bool    isoutatty;		/* is SHOUT a tty */
-bool    isdiagatty;		/* is SHDIAG a tty */
-bool    is1atty;		/* is file descriptor 1 a tty (didfds mode) */
-bool    is2atty;		/* is file descriptor 2 a tty (didfds mode) */
+EXTERN bool    didfds;		/* Have setup i/o fd's for child */
+EXTERN bool    doneinp;		/* EOF indicator after reset from readc */
+EXTERN bool    exiterr;		/* Exit if error or non-zero exit status */
+EXTERN bool    child;		/* Child shell ... errors cause exit */
+EXTERN bool    haderr;		/* Reset was because of an error */
+EXTERN bool    intty;		/* Input is a tty */
+EXTERN bool    intact;		/* We are interactive... therefore prompt */
+EXTERN bool    justpr;		/* Just print because of :p hist mod */
+EXTERN bool    loginsh;		/* We are a loginsh -> .login/.logout */
+EXTERN bool    neednote;	/* Need to pnotify() */
+EXTERN bool    noexec;		/* Don't execute, just syntax check */
+EXTERN bool    pjobs;		/* want to print jobs if interrupted */
+EXTERN bool    setintr;		/* Set interrupts on/off -> Wait intr... */
+EXTERN bool    timflg;		/* Time the next waited for command */
+EXTERN bool    havhash;		/* path hashing is available */
+EXTERN bool    editing;		/* doing filename expansion and line editing */
+EXTERN bool    bslash_quote;	/* PWP: tcsh-style quoting?  (in sh.c) */
+EXTERN bool    isoutatty;	/* is SHOUT a tty */
+EXTERN bool    isdiagatty;	/* is SHDIAG a tty */
+EXTERN bool    is1atty;		/* is file descriptor 1 a tty (didfds mode) */
+EXTERN bool    is2atty;		/* is file descriptor 2 a tty (didfds mode) */
 
 /*
  * Global i/o info
  */
-Char   *arginp;			/* Argument input for sh -c and internal `xx` */
-int     onelflg;		/* 2 -> need line for -t, 1 -> exit on read */
-Char   *ffile;			/* Name of shell file for $0 */
+EXTERN Char   *arginp;		/* Argument input for sh -c and internal `xx` */
+EXTERN int     onelflg;		/* 2 -> need line for -t, 1 -> exit on read */
+EXTERN Char   *ffile;		/* Name of shell file for $0 */
 
 extern char *seterr;		/* Error message from scanner/parser */
 extern int errno;		/* Error from C library routines */
-Char   *shtemp;			/* Temp name for << shell files in /tmp */
+EXTERN Char   *shtemp;		/* Temp name for << shell files in /tmp */
 
 #ifdef BSDTIMES
-struct timeval time0;		/* Time at which the shell started */
-struct rusage ru0;
+EXTERN struct timeval time0;	/* Time at which the shell started */
+EXTERN struct rusage ru0;
 #else
 # ifdef _SEQUENT_
-timeval_t time0;		/* Time at which the shell started */
-struct process_stats ru0;
-# else				/* _SEQUENT_ */
+EXTERN timeval_t time0;		/* Time at which the shell started */
+EXTERN struct process_stats ru0;
+# else /* _SEQUENT_ */
 #  ifndef POSIX
-time_t  time0;			/* time at which shell started */
-#  else				/* POSIX */
-clock_t time0;			/* time at which shell started */
+EXTERN time_t  time0;		/* time at which shell started */
+#  else	/* POSIX */
+EXTERN clock_t time0;		/* time at which shell started */
 #  endif /* POSIX */
-struct tms shtimes;		/* shell and child times for process timing */
+EXTERN struct tms shtimes;	/* shell and child times for process timing */
 # endif /* _SEQUENT_ */
 #endif /* BSDTIMES */
 
 /*
  * Miscellany
  */
-Char   *doldol;			/* Character pid for $$ */
-time_t  chktim;			/* Time mail last checked */
+EXTERN Char   *doldol;		/* Character pid for $$ */
+EXTERN int     backpid;		/* pid of the last background job */
+EXTERN time_t  chktim;		/* Time mail last checked */
 
 /*
  * Ideally these should be uid_t, gid_t, pid_t. I cannot do that right now
@@ -389,14 +429,14 @@ time_t  chktim;			/* Time mail last checked */
  * uid_t and gid_t are not defined in all the systems so I would have to
  * make special cases for them. In the future...
  */
-int     uid;			/* Invokers uid */
-int     gid;			/* Invokers gid */
-int     opgrp,			/* Initial pgrp and tty pgrp */
-        shpgrp,			/* Pgrp of shell */
-        tpgrp;			/* Terminal process group */
+EXTERN int     uid;		/* Invokers uid */
+EXTERN int     gid;		/* Invokers gid */
+EXTERN int     opgrp,		/* Initial pgrp and tty pgrp */
+               shpgrp,		/* Pgrp of shell */
+               tpgrp;		/* Terminal process group */
 				/* If tpgrp is -1, leave tty alone! */
 
-Char    PromptBuf[256];		/* buffer for the actual printed prompt. this
+EXTERN Char    PromptBuf[256];	/* buffer for the actual printed prompt. this
 				 * is used in tenex.c and sh.c for pegets.c */
 
 /*
@@ -407,10 +447,10 @@ Char    PromptBuf[256];		/* buffer for the actual printed prompt. this
  * The desired initial values for these descriptors are defined in
  * sh.local.h.
  */
-int   SHIN;			/* Current shell input (script) */
-int   SHOUT;			/* Shell output */
-int   SHDIAG;			/* Diagnostic output... shell errs go here */
-int   OLDSTD;			/* Old standard input (def for cmds) */
+EXTERN int   SHIN;		/* Current shell input (script) */
+EXTERN int   SHOUT;		/* Shell output */
+EXTERN int   SHDIAG;		/* Diagnostic output... shell errs go here */
+EXTERN int   OLDSTD;		/* Old standard input (def for cmds) */
 
 /*
  * Error control
@@ -429,10 +469,10 @@ extern jmp_buf reslab;
 #define	getexit(a)	copy((char *)(a), (char *)reslab, sizeof reslab)
 #define	resexit(a)	copy((char *)reslab, ((char *)(a)), sizeof reslab)
 
-Char   *gointr;			/* Label for an onintr transfer */
+EXTERN Char   *gointr;		/* Label for an onintr transfer */
 
-sigret_t (*parintr) ();		/* Parents interrupt catch */
-sigret_t (*parterm) ();		/* Parents terminate catch */
+EXTERN sigret_t (*parintr) ();	/* Parents interrupt catch */
+EXTERN sigret_t (*parterm) ();	/* Parents terminate catch */
 
 /*
  * Lexical definitions.
@@ -444,7 +484,7 @@ sigret_t (*parterm) ();		/* Parents terminate catch */
 #define		META		0200
 #define		ASCII		0177
 #ifdef SHORT_STRINGS
-# define	QUOTE 		0100000	/* 16nth char bit used for 'ing */
+# define	QUOTE 	((Char)	0100000)/* 16nth char bit used for 'ing */
 # define	TRIM		0077777	/* Mask to strip quote bit */
 # define	UNDER		0040000	/* Underline flag */
 # define	BOLD		0020000	/* Bold flag */
@@ -453,7 +493,7 @@ sigret_t (*parterm) ();		/* Parents terminate catch */
 # define	ATTRIBUTES	0074000	/* The bits used for attributes */
 # define	CHAR		0000377	/* Mask to mask out the character */
 #else
-# define	QUOTE 		0200	/* Eighth char bit used for 'ing */
+# define	QUOTE 	((Char)	0200)	/* Eighth char bit used for 'ing */
 # define	TRIM		0177	/* Mask to strip quote bit */
 # define	UNDER		0000000	/* No extra bits to do both */
 # define	BOLD		0000000	/* Bold flag */
@@ -463,7 +503,7 @@ sigret_t (*parterm) ();		/* Parents terminate catch */
 # define	CHAR		0000177	/* Mask to mask out the character */
 #endif 
 
-int     AsciiOnly;		/* If set only 7 bits is expected in characters */
+EXTERN int     AsciiOnly;	/* If set only 7 bits expected in characters */
 
 /*
  * Each level of input has a buffered input structure.
@@ -472,7 +512,7 @@ int     AsciiOnly;		/* If set only 7 bits is expected in characters */
  * In other cases, the shell buffers enough blocks to keep all loops
  * in the buffer.
  */
-struct Bin {
+EXTERN struct Bin {
     off_t   Bfseekp;		/* Seek pointer */
     off_t   Bfbobp;		/* Seekp of beginning of buffers */
     off_t   Bfeobp;		/* Seekp of end of buffers */
@@ -480,22 +520,38 @@ struct Bin {
     Char  **Bfbuf;		/* The array of buffer blocks */
 }       B;
 
+/*
+ * This structure allows us to seek inside aliases
+ */
+struct Ain {
+    int type;
+#define I_SEEK -1		/* Invalid seek */
+#define A_SEEK	0		/* Alias seek */
+#define F_SEEK	1		/* File seek */
+#define E_SEEK	2		/* Eval seek */
+    off_t f_seek;
+    Char **a_seek;
+} ;
+
+extern int aret;		/* Type of last char returned */
+#define SEEKEQ(a, b) ((a)->type == (b)->type && \
+		      (a)->f_seek == (b)->f_seek && \
+		      (a)->a_seek == (b)->a_seek)
+
 #define	fseekp	B.Bfseekp
 #define	fbobp	B.Bfbobp
 #define	feobp	B.Bfeobp
 #define	fblocks	B.Bfblocks
 #define	fbuf	B.Bfbuf
 
-#define btell()	fseekp
-
 /*
  * The shell finds commands in loops by reseeking the input
  * For whiles, in particular, it reseeks to the beginning of the
  * line the while was on; hence the while placement restrictions.
  */
-off_t   lineloc;
+EXTERN struct Ain lineloc;
 
-bool    cantell;		/* Is current source tellable ? */
+EXTERN bool    cantell;		/* Is current source tellable ? */
 
 /*
  * Input lines are parsed into doubly linked circular
@@ -527,7 +583,7 @@ struct wordent {
  * process id's from `$$', and modified variable values (from qualifiers
  * during expansion in sh.dol.c) here.
  */
-Char   *lap;
+EXTERN Char   *lap;
 
 /*
  * Parser structure
@@ -635,9 +691,9 @@ extern int nsrchn;
  * source level.  Loops are implemented by seeking back in the
  * input.  For foreach (fe), the word list is attached here.
  */
-struct whyle {
-    off_t   w_start;		/* Point to restart loop */
-    off_t   w_end;		/* End of loop (0 if unknown) */
+EXTERN struct whyle {
+    struct Ain   w_start;	/* Point to restart loop */
+    struct Ain   w_end;		/* End of loop (0 if unknown) */
     Char  **w_fe, **w_fe0;	/* Current/initial wordlist for fe */
     Char   *w_fename;		/* Name for fe */
     struct whyle *w_next;	/* Next (more outer) loop */
@@ -648,7 +704,7 @@ struct whyle {
  *
  * Aliases and variables are stored in AVL balanced binary trees.
  */
-struct varent {
+EXTERN struct varent {
     Char  **vec;		/* Array of words which is the value */
     Char   *v_name;		/* Name of variable/alias */
     struct varent *v_link[3];	/* The links, see below */
@@ -659,7 +715,7 @@ struct varent {
 #define v_right		v_link[1]
 #define v_parent	v_link[2]
 
-struct varent *adrof1();
+extern struct varent *adrof1();
 
 #define adrof(v)	adrof1(v, &shvhed)
 #define value(v)	value1(v, &shvhed)
@@ -668,14 +724,14 @@ struct varent *adrof1();
  * The following are for interfacing redo substitution in
  * aliases to the lexical routines.
  */
-struct wordent *alhistp;	/* Argument list (first) */
-struct wordent *alhistt;	/* Node after last in arg list */
-Char  **alvec;			/* The (remnants of) alias vector */
+EXTERN struct wordent *alhistp;	/* Argument list (first) */
+EXTERN struct wordent *alhistt;	/* Node after last in arg list */
+EXTERN Char  **alvec;		/* The (remnants of) alias vector */
 
 /*
  * Filename/command name expansion variables
  */
-int   gflag;			/* After tglob -> is globbing needed? */
+EXTERN int   gflag;		/* After tglob -> is globbing needed? */
 
 #define MAXVARLEN 30		/* Maximum number of char in a variable name */
 
@@ -698,9 +754,9 @@ extern long gargc;		/* Number args in gargv */
  */
 extern Char **pargv;		/* Pointer to the argv list space */
 extern long pargc;		/* Count of arguments in pargv */
-Char   *pargs;			/* Pointer to start current word */
-long    pnleft;			/* Number of chars left in pargs */
-Char   *pargcp;			/* Current index into pargs */
+EXTERN Char   *pargs;		/* Pointer to start current word */
+EXTERN long    pnleft;		/* Number of chars left in pargs */
+EXTERN Char   *pargcp;		/* Current index into pargs */
 
 /*
  * History list
@@ -713,7 +769,7 @@ Char   *pargcp;			/* Current index into pargs */
  * when history substitution includes modifiers, and thrown away
  * at the next discarding since their event numbers are very negative.
  */
-struct Hist {
+EXTERN struct Hist {
     struct wordent Hlex;
     int     Hnum;
     int     Href;
@@ -722,12 +778,12 @@ struct Hist {
     struct Hist *Hnext;
 }       Histlist;
 
-struct wordent paraml;		/* Current lexical word list */
-int     eventno;		/* Next events number */
-int     lastev;			/* Last event reference (default) */
+EXTERN struct wordent paraml;	/* Current lexical word list */
+EXTERN int     eventno;		/* Next events number */
+EXTERN int     lastev;		/* Last event reference (default) */
 
-Char    HIST;			/* history invocation character */
-Char    HISTSUB;		/* auto-substitute character */
+EXTERN Char    HIST;		/* history invocation character */
+EXTERN Char    HISTSUB;		/* auto-substitute character */
 
 /*
  * To print system call errors...
@@ -735,12 +791,6 @@ Char    HISTSUB;		/* auto-substitute character */
 extern char *sys_errlist[];
 extern int errno, sys_nerr;
 
-#ifdef strerror
-# undef strerror
-#endif 
-#define strerror(e) ((e) < sys_nerr && (e) >= 0 ? sys_errlist[(e)] :\
-		"Unknown Error")
-
 /*
  * strings.h:
  */
@@ -785,20 +835,19 @@ extern int errno, sys_nerr;
 /*
  * setname is a macro to save space (see sh.err.c)
  */
-char   *bname;
+EXTERN char   *bname;
 
 #define	setname(a)	(bname = (a))
 
 #ifdef VFORK
-Char   *Vsav;
-Char   *Vdp;
-Char   *Vexpath;
-char  **Vt;
-
+EXTERN Char   *Vsav;
+EXTERN Char   *Vdp;
+EXTERN Char   *Vexpath;
+EXTERN char  **Vt;
 #endif /* VFORK */
 
-Char  **evalvec;
-Char   *evalp;
+EXTERN Char  **evalvec;
+EXTERN Char   *evalp;
 
 extern struct mesg {
     char   *iname;		/* name from /usr/include */
@@ -808,17 +857,17 @@ extern struct mesg {
 /* word_chars is set by default to WORD_CHARS but can be overridden by
    the worchars variable--if unset, reverts to WORD_CHARS */
 
-Char   *word_chars;
+EXTERN Char   *word_chars;
 
 #define WORD_CHARS "*?_-.[]~="	/* default chars besides alnums in words */
 
-Char   *STR_SHELLPATH;
+EXTERN Char   *STR_SHELLPATH;
 
 #ifdef _PATH_BSHELL
-Char   *STR_BSHELL;
+EXTERN Char   *STR_BSHELL;
 #endif 
-Char   *STR_WORD_CHARS;
-Char  **STR_environ;
+EXTERN Char   *STR_WORD_CHARS;
+EXTERN Char  **STR_environ;
 
 #include "tc.h"
 #include "sh.decls.h"
