@@ -1,4 +1,4 @@
-/* $Header: /home/hyperion/mu/christos/src/sys/tcsh-6.00/RCS/sh.misc.c,v 3.4 1991/07/15 19:37:24 christos Exp $ */
+/* $Header: /home/hyperion/mu/christos/src/sys/tcsh-6.00/RCS/sh.misc.c,v 3.7 1991/10/12 04:23:51 christos Exp $ */
 /*
  * sh.misc.c: Miscelaneous functions
  */
@@ -34,11 +34,10 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  */
-#include "config.h"
-RCSID("$Id: sh.misc.c,v 3.4 1991/07/15 19:37:24 christos Exp $")
-
 #include "sh.h"
 
+RCSID("$Id: sh.misc.c,v 3.7 1991/10/12 04:23:51 christos Exp $")
+
 static	int	renum	__P((int, int));
 
 /*
@@ -319,7 +318,7 @@ dcopy(i, j)
     register int i, j;
 {
 
-    if (i == j || i < 0 || j < 0 && i > 2)
+    if (i == j || i < 0 || (j < 0 && i > 2))
 	return (i);
 #ifdef HAVEDUP2
     if (j >= 0) {
@@ -371,10 +370,10 @@ lshift(v, c)
     register Char **v;
     register int c;
 {
-    register Char **u = v;
+    register Char **u;
 
-    while (*u && --c >= 0)
-	xfree((ptr_t) * u++);
+    for (u = v; *u && --c >= 0; u++)
+	xfree((ptr_t) *u);
     (void) blkcpy(v, u);
 }
 
