@@ -1,4 +1,4 @@
-/* $Header: /home/hyperion/mu/christos/src/sys/tcsh-6.00/RCS/sh.proc.c,v 3.9 1991/08/05 23:02:13 christos Exp $ */
+/* $Header: /home/hyperion/mu/christos/src/sys/tcsh-6.00/RCS/sh.proc.c,v 3.14 1991/10/20 01:38:14 christos Exp $ */
 /*
  * sh.proc.c: Job manipulations
  */
@@ -34,39 +34,13 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  */
-#include "config.h"
-RCSID("$Id: sh.proc.c,v 3.9 1991/08/05 23:02:13 christos Exp $")
-
 #include "sh.h"
+
+RCSID("$Id: sh.proc.c,v 3.14 1991/10/20 01:38:14 christos Exp $")
+
 #include "ed.h"
 #include "tc.h"
-
-/*
- * a little complicated #include <sys/wait.h>! :-(
- */
-#if SVID > 0
-# ifdef hpux
-#  ifndef __hpux
-#   include "tc.wait.h"	/* 6.5 broke <sys/wait.h> */
-#  else
-#   ifndef POSIX
-#    define _BSD
-#   endif
-#   ifndef _CLASSIC_POSIX_TYPES
-#    define _CLASSIC_POSIX_TYPES
-#   endif
-#   include <sys/wait.h> /* 7.0 fixed it again */
-#  endif /* __hpux */
-# else /* hpux */
-#  if defined(OREO) || defined(IRIS4D) || defined(POSIX)
-#   include <sys/wait.h>
-#  else	/* OREO || IRIS4D || POSIX */
-#   include "tc.wait.h"
-#  endif /* OREO || IRIS4D || POSIX */
-# endif	/* hpux */
-#else /* SVID == 0 */
-# include <sys/wait.h>
-#endif /* SVID == 0 */
+#include "tc.wait.h"
 
 #if !defined(NSIG) && defined(SIGMAX)
 # define NSIG (SIGMAX+1)
@@ -81,7 +55,7 @@ RCSID("$Id: sh.proc.c,v 3.9 1991/08/05 23:02:13 christos Exp $")
 # define HZ	100		/* for division into seconds */
 #endif
 
-#if (defined(_BSD) && defined(_BSD_INCLUDES))
+#if (defined(_BSD) && defined(_BSD_INCLUDES)) || (defined(IRIS4D) && __STDC__)
 # define BSDWAIT
 #endif
 #ifndef WTERMSIG
@@ -221,7 +195,7 @@ loop:
 #ifdef BSDJOBS
 # ifdef BSDTIMES
     /* both a wait3 and rusage */
-#  ifndef BSDWAIT
+#  if !defined(BSDWAIT) || defined(NeXT) || (defined(IRIS4D) && __STDC__)
     pid = wait3(&w,
        (setintr && (intty || insource) ? WNOHANG | WUNTRACED : WNOHANG), &ru);
 #  else /* BSDWAIT */
@@ -255,17 +229,29 @@ loop:
     pid = wait3(&w.w_status, WNOHANG, &ru);
 #  endif /* !hpux */
 # else /* !BSDTIMES */
-#  if SVID < 3
+# ifdef ODT  /* For Sco Unix 3.2.0 or ODT 1.0 */
+#  define HAVEwait
+    pid = waitpid(-1, &w,
+	    (setintr && (intty || insource) ? WNOHANG | WUNTRACED : WNOHANG));
+# endif /* ODT */	    
+# ifdef aiws
+#  define HAVEwait
+    pid = wait3(&w.w_status, 
+	(setintr && (intty || insource) ? WNOHANG | WUNTRACED : WNOHANG), 0);
+# endif /* aiws */
+# ifndef HAVEwait
+#  ifdef UNRELSIGS
     /* no wait3, therefore no rusage */
     /* on Sys V, this may hang.  I hope it's not going to be a problem */
     pid = ourwait(&w.w_status);
-#  else	/* SVID >= 3 */
+#  else	/* UNRELSIGS */
     /* 
      * XXX: for greater than 3 we should use waitpid(). 
      * but then again, SVR4 falls into the POSIX/BSDJOBS category.
      */
     pid = wait(&w.w_status);
 #  endif /* SVID >= 3 */
+# endif /* HAVEwait */
 # endif	/* BSDTIMES */
 # ifndef BSDSIGS
     (void) sigset(SIGCHLD, pchild);
@@ -443,9 +429,9 @@ found:
 	    }
 	}
     }
-#ifdef BSDJOBS
+#if defined(BSDJOBS) || defined(WNOHANG)
     goto loop;
-#endif /* BSDJOBS */
+#endif /* BSDJOBS || WNOHANG */
 }
 
 void
@@ -534,9 +520,9 @@ pjwait(pp)
 #ifdef BSDSIGS
     sigmask_t omask;
 #endif /* BSDSIGS */
-#if (SVID > 0) && (SVID < 3)
+#ifdef UNRELSIGS
     sigret_t (*inthandler)();
-#endif /* (SVID > 0) && (SVID < 3) */
+#endif /* UNRELSIGS */
 
     while (pp->p_pid != pp->p_jobid)
 	pp = pp->p_friends;
@@ -554,10 +540,10 @@ pjwait(pp)
 #ifdef BSDSIGS
     omask = sigblock(sigmask(SIGCHLD));
 #endif /* BSDSIGS */
-#if (SVID > 0) && (SVID < 3)
+#ifdef UNRELSIGS
     if (setintr)
         inthandler = signal(SIGINT, SIG_IGN);
-#endif /* (SVID > 0) && (SVID < 3) */
+#endif /* UNRELSIGS */
     for (;;) {
 #ifndef BSDSIGS
 	(void) sighold(SIGCHLD);
@@ -583,10 +569,10 @@ pjwait(pp)
 #else /* !BSDSIGS */
     (void) sigrelse(SIGCHLD);
 #endif /* !BSDSIGS */
-#if (SVID > 0) && (SVID < 3)
+#ifdef UNRELSIGS
     if (setintr)
         (void) signal(SIGINT, inthandler);
-#endif /* (SVID > 0) && (SVID < 3) */
+#endif /* UNRELSIGS */
 #ifdef BSDJOBS
     if (tpgrp > 0)		/* get tty back */
 	(void) tcsetpgrp(FSHTTY, tpgrp);
@@ -972,6 +958,14 @@ pendjob()
 /*
  * pprint - print a job
  */
+
+/*
+ * Hacks have been added for SVR4 to deal with pipe's being spawned in
+ * reverse order
+ *
+ * David Dawes (dawes@physics.su.oz.au) Oct 1991
+ */
+
 static int
 pprint(pp, flag)
     register struct process *pp;
@@ -980,9 +974,14 @@ pprint(pp, flag)
     register status, reason;
     struct process *tp;
     extern char *linp, linbuf[];
-    int     jobflags, pstatus;
+    int     jobflags, pstatus, pcond;
     char   *format;
 
+#ifdef BACKPIPE
+    struct process *pipehead, *pipetail, *pmarker;
+    int inpipe = 0;
+#endif /* BACKPIPE */
+
     while (pp->p_pid != pp->p_jobid)
 	pp = pp->p_friends;
     if (pp == pp->p_friends && (pp->p_flags & PPTIME)) {
@@ -993,23 +992,53 @@ pprint(pp, flag)
     status = reason = -1;
     jobflags = 0;
     do {
+#ifdef BACKPIPE
+	/*
+	 * The pipeline is reversed, so locate the real head of the pipeline
+	 * if pp is at the tail of a pipe (and not already in a pipeline)
+	 */
+	if ((pp->p_friends->p_flags & PPOU) && !inpipe && (flag & NAME)) {
+	    inpipe = 1;
+	    pipetail = pp;
+	    do 
+		pp = pp->p_friends;
+	    while (pp->p_friends->p_flags & PPOU);
+	    pipehead = pp;
+	    pmarker = pp;
+	/*
+	 * pmarker is used to hold the place of the proc being processed, so
+	 * we can search for the next one downstream later.
+	 */
+	}
+	pcond = (tp != pp || (inpipe && tp == pp));
+#else /* !BACKPIPE */
+	pcond = (tp != pp);
+#endif /* BACKPIPE */	    
+
 	jobflags |= pp->p_flags;
 	pstatus = pp->p_flags & PALLSTATES;
-	if (tp != pp && linp != linbuf && !(flag & FANCY) &&
-	    (pstatus == status && pp->p_reason == reason ||
+	if (pcond && linp != linbuf && !(flag & FANCY) &&
+	    ((pstatus == status && pp->p_reason == reason) ||
 	     !(flag & REASON)))
 	    xprintf(" ");
 	else {
-	    if (tp != pp && linp != linbuf)
+	    if (pcond && linp != linbuf)
 		xprintf("\n");
-	    if (flag & NUMBER)
-		if (pp == tp)
+	    if (flag & NUMBER) {
+#ifdef BACKPIPE
+		pcond = ((pp == tp && !inpipe) ||
+			 (inpipe && pipetail == tp && pp == pipehead));
+#else /* BACKPIPE */
+		pcond = (pp == tp);
+#endif /* BACKPIPE */
+		if (pcond)
 		    xprintf("[%d]%s %c ", pp->p_index,
 			    pp->p_index < 10 ? " " : "",
 			    pp == pcurrent ? '+' :
 			    (pp == pprevious ? '-' : ' '));
 		else
 		    xprintf("       ");
+	    }
 	    if (flag & FANCY) {
 #ifdef TCF
 		extern char *sitename();
@@ -1118,7 +1147,13 @@ prcomd:
 #endif /* !BSDTIMES && !SEQUENT */
 
 	}
-	if (tp == pp->p_friends) {
+#ifdef BACKPIPE
+	pcond = ((tp == pp->p_friends && !inpipe) ||
+		 (inpipe && pipehead->p_friends == tp && pp == pipetail));
+#else  /* !BACKPIPE */
+	pcond = (tp == pp->p_friends);
+#endif /* BACKPIPE */
+	if (pcond) {
 	    if (linp != linbuf)
 		xprintf("\n");
 	    if (flag & SHELLDIR && !eq(tp->p_cwd->di_name, dcwd->di_name)) {
@@ -1127,7 +1162,33 @@ prcomd:
 		xprintf(")\n");
 	    }
 	}
-    } while ((pp = pp->p_friends) != tp);
+#ifdef BACKPIPE
+	if (inpipe) {
+	    /*
+	     * if pmaker == pipetail, we are finished that pipeline, and
+	     * can now skip to past the head
+	     */
+	    if (pmarker == pipetail) {
+		inpipe = 0;
+		pp = pipehead;
+	    }
+	    else {
+	    /*
+	     * set pp to one before the one we want next, so the while below
+	     * increments to the correct spot.
+	     */
+		do
+		    pp = pp->p_friends;
+	    	while (pp->p_friends->p_friends != pmarker);
+	    	pmarker = pp->p_friends;
+	    }
+	}
+	pcond = ((pp = pp->p_friends) != tp || inpipe);
+#else /* !BACKPIPE */
+	pcond = ((pp = pp->p_friends) != tp);
+#endif /* BACKPIPE */
+    } while (pcond);
+
     if (jobflags & PTIME && (jobflags & (PSTOPPED | PRUNNING)) == 0) {
 	if (jobflags & NUMBER)
 	    xprintf("       ");
@@ -1672,7 +1733,7 @@ pfork(t, wanttty)
 #ifdef BSDSIGS
     sigmask_t omask;
 #endif /* BSDSIGS */
-#if SIGSYNCH
+#ifdef SIGSYNCH
     sigvec_t osv;
     static sigvec_t nsv = {synch_handler, ~0, 0};
 #endif /* SIGSYNCH */
@@ -1857,6 +1918,12 @@ pgetty(wanttty, pgrp)
 	(void) sighold(SIGTTOU);
     }
 #  endif /* !BSDSIGS */
+# endif /* POSIXJOBS */
+
+# ifndef POSIXJOBS
+    if (wanttty > 0)
+	(void) tcsetpgrp(FSHTTY, pgrp);
+# endif /* !POSIXJOBS */
 
     /*
      * From: Michael Schroeder <mlschroe@immd4.informatik.uni-erlangen.de>
@@ -1866,37 +1933,37 @@ pgetty(wanttty, pgrp)
      */
     if (wanttty >= 0)
 	if (setpgid(0, pgrp) == -1) {
-#  if !defined(ISC) && !defined(SCO)
+# ifdef BACKPIPE
+	    /*
+     	     * This usually happens in svr4 when the last command in a pipe
+	     * either couldn't be started, or exits without waiting for input.
+	     * Putting in the xexit() hangs the shell, so leave it out.
+	     * (DHD)
+     	     */
+#  ifdef JOBDEBUG
+	    xprintf("tcsh: setpgid error (%s).\n", strerror(errno));
+	    xprintf("pgrp = %d, shell pid = %d\n",pgrp,getpid());
+#  endif /* JOBDEBUG */
+# else /* !BACKPIPE */
+#  if !defined(ISC) && !defined(SCO) && !defined(cray)
 	    /* XXX: Wrong but why? */
 	    xprintf("tcsh: setpgid error (%s).\n", strerror(errno));
+#  endif /* !ISC && !SCO && !cray */
 	    xexit(0);
-#  endif /* !ISC && !SCO */
+# endif /* BACKPIPE */
 	}
-# endif /* POSIXJOBS */
 
-    if (wanttty > 0)
+# ifdef POSIXJOBS
+    if (wanttty > 0) {
 	(void) tcsetpgrp(FSHTTY, pgrp);
-
-# ifndef POSIXJOBS
-    if (wanttty >= 0)
-	if (setpgid(0, pgrp) == -1) {
-#  if !defined(ISC) && !defined(SCO)
-	    /* XXX: Wrong but why? */
-	    xprintf("tcsh: setpgid error (%s).\n", strerror(errno));
-	    xexit(0);
-#  endif /* !ISC && !SCO */
-	}
-# else /* POSIXJOBS */
-    if (wanttty > 0)
 #  ifdef BSDSIGS
 	(void) sigsetmask(omask);
 #  else /* BSDSIGS */
-    {
 	(void) sigrelse(SIGTSTP);
 	(void) sigrelse(SIGTTIN);
 	(void) sigrelse(SIGTTOU);
-    }
 #  endif /* !BSDSIGS */
+    }
 # endif /* POSIXJOBS */
 
     if (tpgrp > 0)
