@@ -1,4 +1,4 @@
-/* $Header: /home/hyperion/mu/christos/src/sys/tcsh-6.00/RCS/sh.types.h,v 3.9 1991/08/06 01:49:40 christos Exp $ */
+/* $Header: /home/hyperion/mu/christos/src/sys/tcsh-6.00/RCS/sh.types.h,v 3.12 1991/10/18 16:27:13 christos Exp $ */
 /* sh.types.h: Do the necessary typedefs for each system.
  *             Up till now I avoided making this into a separate file
  *	       But I just wanted to eliminate the whole mess from sh.h
@@ -128,7 +128,7 @@ extern uid_t getuid(), geteuid();
 extern gid_t getgid(), getegid();
 extern sigmask_t sigblock();
 extern sigmask_t sigsetmask();
-#ifndef __hp9000s700
+#ifdef notdef	/* XXX: Not true for 8.0 */
 extern void sigpause();
 extern sigmask_t sigspace();
 #endif
@@ -141,7 +141,7 @@ extern void qsort();
 extern void free();
 extern unsigned int alarm();
 extern unsigned int sleep();
-#ifndef __hp9000s700
+#ifdef notdef	/* XXX: Not true for 8.0 */
 extern int lstat();
 extern int readlink();
 extern int sigvector();
@@ -240,15 +240,17 @@ extern char *sbrk();
 
 
 /***
- *** rs6000, ibm370, ps2: running flavors of aix.
+ *** rs6000, ibm370, ps2, rt: running flavors of aix.
  ***/
 #ifdef IBMAIX
 # ifndef _SIZE_T
 #  define _SIZE_T
 # endif /* _SIZE_T */
-# ifndef _PID_T
-#  define _PID_T
-# endif /* _PID_T */
+# ifndef aiws
+#  ifndef _PID_T
+#   define _PID_T
+#  endif /* _PID_T */
+# endif /* !aiws */
 # ifdef _IBMR2
 #  ifndef _SPEED_T 
 #   define _SPEED_T
@@ -260,7 +262,7 @@ extern char *sbrk();
 /***
  *** Ultrix...
  ***/
-#ifdef ultrix
+#if defined(ultrix) || defined(__ultrix)
 # ifndef _SIZE_T
 #  define _SIZE_T
 # endif /* _SIZE_T */
@@ -353,16 +355,51 @@ extern char *sbrk();
 # endif /* _PID_T */
 #endif /* uts */
 
+/*
+ * UnixPC aka u3b1
+ */
+#ifdef UNIXPC
+# ifdef types_h
+#  ifndef _SIZE_T
+#   define _SIZE_T
+#  endif /* _SIZE_T */
+# endif /* types_h */
+#endif /* UNIXPC */
+
+/*
+ * NS32000 OPUS
+ */
+#ifdef OPUS
+# ifndef _SIZE_T
+#  define _SIZE_T
+# endif /* _SIZE_T */
+#endif /* OPUS */
+
+/*
+ * Convex
+ */
+#if defined(convex) || defined(__convex__)
+# if defined(__SIZE_T) && !defined(_SIZE_T)
+#  define _SIZE_T
+# endif /* __SIZE_T && !_SIZE_T */
+#endif /* convex || __convex__ */
+
 /***
- *** Catch all for non POSIX systems.
- *** Posix things *should* define these automatically
+ *** Catch all for non POSIX and/or non ANSI systems.
+ *** Systems up to spec *should* define these automatically
  *** I am open to suggestions on how to do this correctly!
  ***/
-#ifndef POSIX
+
+#ifndef __STDC__
+
 # ifndef _SIZE_T
 #  define _SIZE_T
    typedef int size_t;		/* As sun comments ??? : meaning I take it */
-# endif /* _SIZE_T */		/* Until we make the world POSIX... */
+# endif /* _SIZE_T */		/* Until we make the world ANSI... */
+
+#endif  /* ! __STDC__ */
+
+#ifndef POSIX
 
 # ifndef _PID_T
 #  define _PID_T
