@@ -1,4 +1,4 @@
-/* $Header: /tmp_mnt/home/ragmanns/mu/christos/src/sys/tcsh-6.00/RCS/tc.alloc.c,v 3.3 91/07/19 01:31:14 christos Exp $ */
+/* $Header: /home/hyperion/mu/christos/src/sys/tcsh-6.00/RCS/tc.alloc.c,v 3.6 1991/10/12 04:23:51 christos Exp $ */
 /*
  * tc.alloc.c (Caltech) 2/21/82
  * Chris Kingsley, kingsley@cit-20.
@@ -42,12 +42,10 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  */
-#include "config.h"
-RCSID("$Id: tc.alloc.c,v 3.3 91/07/19 01:31:14 christos Exp $")
-
-
 #include "sh.h"
 
+RCSID("$Id: tc.alloc.c,v 3.6 1991/10/12 04:23:51 christos Exp $")
+
 char   *memtop = NULL;		/* PWP: top of current memory */
 char   *membot = NULL;		/* PWP: bottom of allocatable memory */
 
@@ -130,21 +128,19 @@ static	void	morecore	__P((int));
 
 
 #ifdef DEBUG
-#define CHECK(a, str, p) \
+# define CHECK(a, str, p) \
     if (a) { \
 	xprintf(str, p);	\
 	xprintf("memtop = %lx membot = %lx.\n", memtop, membot);	\
 	abort(); \
-    }	\
-    else
+    }
 #else
-#define CHECK(a, str, p) \
+# define CHECK(a, str, p) \
     if (a) { \
 	xprintf(str, p);	\
 	xprintf("memtop = %lx membot = %lx.\n", memtop, membot);	\
 	return; \
-    }	\
-    else
+    }
 #endif
 
 memalign_t
@@ -324,7 +320,7 @@ calloc(i, j)
 	    *cp++ = 0;
 	while (--i);
 
-    return (scp);
+    return ((memalign_t) scp);
 #else
     if (i && j)
 	return ((memalign_t) 0);
@@ -410,15 +406,16 @@ realloc(cp, nbytes)
     onb = MEMALIGN(nbytes + MEMALIGN(sizeof(union overhead)) + RSLOP);
 
     /* avoid the copy if same size block */
-    if (was_alloced && (onb < (1 << (i + 3))) && (onb >= (1 << (i + 2))))
+    if (was_alloced && (onb < (U_int) (1 << (i + 3))) && 
+	(onb >= (U_int) (1 << (i + 2))))
 	return ((memalign_t) cp);
     if ((res = malloc(nbytes)) == NULL)
-	return (NULL);
+	return ((memalign_t) NULL);
     if (cp != res)		/* common optimization */
 	mybcopy(cp, res, nbytes);
     if (was_alloced)
 	free(cp);
-    return (res);
+    return ((memalign_t) res);
 #else
     if (cp && nbytes)
 	return ((memalign_t) 0);
