@@ -1,4 +1,4 @@
-/* $Header: /home/hyperion/mu/christos/src/sys/tcsh-6.00/RCS/tc.const.c,v 3.3 1991/08/05 23:02:13 christos Exp $ */
+/* $Header: /home/hyperion/mu/christos/src/sys/tcsh-6.00/RCS/tc.const.c,v 3.4 1991/10/12 04:23:51 christos Exp $ */
 /*
  * sh.const.c: String constants for tcsh.
  */
@@ -34,10 +34,10 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  */
-#include "config.h"
-RCSID("$Id: tc.const.c,v 3.3 1991/08/05 23:02:13 christos Exp $")
 #include "sh.h"
 
+RCSID("$Id: tc.const.c,v 3.4 1991/10/12 04:23:51 christos Exp $")
+
 Char STRlogout[]        = { 'l', 'o', 'g', 'o', 'u', 't', '\0' };
 Char STRautologout[]    = { 'a', 'u', 't', 'o', 'l', 'o', 'g', 'o', 'u', 't', 
 			    '\0' };
@@ -96,8 +96,8 @@ Char STRsymhash[]       = { '#', ' ', '\0' };
 Char STRsymarrow[]      = { '>', ' ', '\0' };
 Char STRmquestion[]     = { '%', 'R', '?' | QUOTE, ' ', '\0' };
 Char STRCORRECT[]       = { 'C', 'O', 'R', 'R', 'E', 'C', 'T', '>', '%', 'R', 
-			    ' ', '(', 'y', '|', 'n', ')', '?' | QUOTE, ' ', 
-			    '\0' };
+			    ' ', '(', 'y', '|', 'n', '|', 'e', ')', 
+			    '?' | QUOTE, ' ', '\0' };
 Char STRunalias[]       = { 'u', 'n', 'a', 'l', 'i', 'a', 's', '\0' };
 Char STRalias[]         = { 'a', 'l', 'i', 'a', 's', '\0' };
 Char STRprecmd[]        = { 'p', 'r', 'e', 'c', 'm', 'd', '\0' };
